#include <string.h>
#include <stdio.h>
#include <stdlib.h>

typedef int data;
/*
 * utilizzzo i 5 bit meno significativi per il giorno, i 4 successivi per il mese, i rimanenti per l'anno
 * vedi nuova_data
 * in questo modo il confronto fra date  pi semplice
 */

static char *strmesi[] = { "???",
	"gen", "feb", "mar", "apr", "mag", "giu",
	"lug", "ago", "set", "ott", "nov", "dic" };

/*
 * definisco un tipo particolare per i mesi
 * e una serie di costanti associate - i valori partono da 0 e si incremnetano di 1
 * perci ERR == 0 ... DIC == 12
 */
typedef enum { ERR, GEN, FEB, MAR, APR, MAG, GIU, LUG, AGO, SET, OTT, NOV, DIC }  mesi;
// non viene fatto comunque nessun controllo particolare, qualunque intero  comunque accettato

mesi a2mesi(char *mese)
{
	mesi i;
	for(i=1; i<=DIC; i++) {
		if(strcmp(mese, strmesi[i])==0) return i;
	}
	return ERR;
}

#define GIORNO(g) ((g) & 31)
#define MESE(m) (((m)>>5) & 15)
#define ANNO(a) ((a)>>9)

char* data2str(data d)
{
	static char buffer[12]; // buffer viene allocato in una regione di memoria fissa
	/*
	 * il valore memorizzato si conserva da una chiamata alla successiva
	 */
	sprintf(buffer, "%2d %3s %4d", GIORNO(d), strmesi[MESE(d)], ANNO(d));
	return buffer;
}

data nuova_data(int giorno, char* mese, int anno)
{
	return giorno | (a2mesi(mese)<<5) | (anno<<9);
}

typedef struct {
	int size;
	char* owner;
	data d;
	char *nome;
} file_info;

int cmp(file_info *f1, file_info *f2)
{
	return f1->d-f2->d;
}

#define TEST(X)	do { if((X)==NULL) { fprintf(stderr, "TEST failed: function %s, line %d\n", __func__, __LINE__); exit(1); } } while(0)

void a2info(char *s, file_info *f)
{
	char nome[100], owner[100], mese[4];
	int giorno, anno;
	sscanf(s, "%d %s %d %s %d %s", &f->size, owner, &giorno, mese, &anno, nome);
	TEST(f->owner = strdup(owner));
	f->d = nuova_data(giorno, mese, anno);
	TEST(f->nome = strdup(nome));
}

void stampa_info(file_info *inf, FILE *f)
{
	fprintf(f, "%5d %-10s %s %s\n", inf->size, inf->owner, data2str(inf->d), inf->nome);
}

typedef struct {
	int n;
	file_info *elenco;
} dir_info;

void leggi_dir(dir_info *inf, FILE *f)
{
	char buffer[200];
	TEST(fgets(buffer, sizeof(buffer), f));
	inf->n = 0;
	sscanf(buffer, "Numero di file: %d", &inf->n);
	TEST(inf->elenco = calloc(inf->n, sizeof(*inf->elenco)));
	int i;
	for(i=0; i<inf->n; i++) {
		TEST(fgets(buffer, sizeof(buffer), f));
		a2info(buffer, inf->elenco+i);
	}
}

void stampa_dir(dir_info *inf, FILE *f)
{
	fprintf(f, "Numero di file: %d\n", inf->n);
	int i;
	for(i=0; i<inf->n; i++) {
		stampa_info(inf->elenco+i, f);
	}
}

void bubblesort(dir_info *inf)
{
	file_info *vet = inf->elenco;
	int i, j, k;
	for(j=inf->n-1; j>0; j=k) {
		k=-1;
		for(i=0; i<j; i++) {
			if(cmp(vet+i, vet+i+1) > 0) {
				typeof(*vet) tmp = vet[i];
				vet[i] = vet[i+1];
				vet[i+1] = tmp;
				k=i;
			}
		}
	}
}

/*
 * l'algoritmo pu essere ottimizzato se il vettore
 * fosse sempre ordinato per data
 */
int occupazione(dir_info *inf, data d)
{
	int somma = 0, i;
	for(i=0; i<inf->n; i++) {
		if(inf->elenco[i].d>d) somma += inf->elenco[i].size;
	}
	return somma;
}

int main(int argc, char *argv[])
{
	dir_info inf;

	// gestione parametri
	FILE *fin = stdin, *fout=stdout;
	if(argc>1) TEST(fin = fopen(argv[1], "r"));
	if(argc>2) TEST(fout = fopen(argv[2], "w"));

	leggi_dir(&inf, fin);
	stampa_dir(&inf, stderr); // stampa di controllo, non richiesta

	// considero tutti i file dall'anno 0
	fprintf(stdout, "Occupazione totale: %d\n", occupazione(&inf, 0));

	bubblesort(&inf);
	stampa_dir(&inf, fout);

	// & 0xffffffe0 azzero gli ultimi 5 bit - cio il giorno
	fprintf(stdout, "Occupazione ultimo mese: %d\n", occupazione(&inf, inf.elenco[inf.n-1].d & 0xffffffe0));
	return 0;
}

#if 0
esempio-dati
Numero di file: 20
91030 leone      11 feb 2015 Dato.java
35119 ciampi     22 gen 2015 Dato.class
  666 scalfaro   17 apr 2015 Esame.java
  560 ciampi     30 mag 2015 Esame.class
84324 scalfaro   25 mar 2015 Esame.html
91825 leone       4 set 2015 esempio
85018 cossiga    28 lug 2015 esempio.txt
 1509 pertini    23 mag 2015 lista.txt
78477 saragat    28 gen 2015 Lista.java
97770 leone       3 mar 2015 Lista.class
18495 scalfaro    7 giu 2015 Lista.html
63664 saragat    21 mag 2015 index.html
 4078 ciampi     21 mar 2015 tmp012.txt
49265 cossiga    14 mag 2015 tmp013.txt
23905 ciampi     15 mar 2015 tmp014.txt
37824 gronchi    24 mag 2015 tesi.doc
62215 scalfaro    7 ago 2015 tesi.pdf
 2716 ciampi      6 set 2015 introduzione.txt
20701 napolitano 10 set 2015 introduzione.pdf
37100 napolitano 14 set 2015 file0019.tmp
esempio-dati
#endif
