#include <stdio.h>
#include <stdlib.h>

typedef struct {
	int nrig, ncol, max;
	unsigned int **mat;
} immagine, *pimmagine;

// la matrice  azzerata
void imm_alloca(pimmagine m)
{
	m->mat = malloc(m->nrig * sizeof(*m->mat));
	if (m->mat == NULL) {
		perror(__func__);
		exit(1);
	}
	m->mat[0] = calloc(m->nrig * m->ncol, sizeof(**m->mat));
	if (m->mat[0] == NULL) {
		perror(__func__);
		exit(1);
	}
	int i;
	for (i = 1; i < m->nrig; i++) {
    	m->mat[i] = m->mat[i-1]+m->ncol;
	}
}

void imm_stampa(pimmagine mat, FILE* f)
{
	int i, j;
	fprintf(f, "P2\n%d %d\n%d\n", mat->ncol, mat->nrig, mat->max);
	for(i=0; i<mat->nrig; i++) {
		for(j=0; j<mat->ncol; j++) {
			fprintf(f, "%3d ", mat->mat[i][j]);
		}
		fprintf(f, "\n");
	}
	fprintf(f, "\n");
}

void imm_leggi(pimmagine mat, FILE* f)
{
	int i, j;
	if(fscanf(f, "P2 %d %d %d", &mat->ncol, &mat->nrig, &mat->max)!=3) {
		fprintf(stderr, "%s: intestazione file non corretta!\n", __func__);
		exit(1);
	}
	imm_alloca(mat);
	for(i=0; i<mat->nrig; i++) {
		for(j=0; j<mat->ncol; j++) {
			fscanf(f, "%ud ", &mat->mat[i][j]);
		}
	}
}

void imm_leggi_da_file(pimmagine mat, char *fn)
{
	FILE *f = fopen(fn, "r");
	if(!f) {
		perror(fn);
		exit(1);
	}
	imm_leggi(mat, f);
}

void imm_stampa_su_file(pimmagine mat, char *fn)
{
	FILE *f = fopen(fn, "w");
	if(!f) {
		perror(fn);
		exit(1);
	}
	imm_stampa(mat, f);
}

/*
 * se il puntatore nn  nullo il numero di pixel non
 * viene restituito
 */
double imm_media(pimmagine mat, int *nn)
{
	int i, j, n=0, somma = 0;
	for(i=0; i<mat->nrig; i++) {
		for(j=0; j<mat->ncol; j++) {
			int val = mat->mat[i][j];
			if(val==0) continue;
			somma += val;
			++n;
		}
	}
	if(nn) *nn = n;
	return somma/(double) n;
}

immagine imm_contorno(pimmagine mat)
{
	int i, j;
	immagine cont = *mat;
	imm_alloca(&cont);
	for(i=1; i<mat->nrig-1; i++) {
		for(j=1; j<mat->ncol-1; j++) {
			if(mat->mat[i][j]==0) continue;
			if(mat->mat[i-1][j]==0 || mat->mat[i][j-1]==0 || mat->mat[i][j+1]==0 || mat->mat[i+1][j]==0) {
				cont.mat[i][j] = mat->mat[i][j];
			}
		}
	}
	return cont;
}

int main(int argc, char *argv[])
{
	immagine mat;
	if(argc>1) {
		imm_leggi_da_file(&mat, argv[1]);
	} else {
		imm_leggi_da_file(&mat, "esempio.pgm");
	}
	int cont;
	double m = imm_media(&mat, &cont);
	mat = imm_contorno(&mat);
	printf("Valor medio: %f, numero pixel: %d\n", m, cont);

	if(argc>2) {
		imm_stampa_su_file(&mat, argv[2]);
	} else {
		imm_stampa(&mat, stdout);
	}
	return 0;
}

#if 0
esempio-dati
P2
30 20
255
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 255   0   0   0   0   0   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0 249 244 242 242 242 244 249   0   0   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0 252 244 236 231 229 229 229 231 236 244 252   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0 252 242 234 226 218 216 216 216 218 226 234 242 252   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0 244 234 224 216 208 203 203 203 208 216 224 234 244   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0 249 236 226 216 206 198 190 190 190 198 206 216 226 236 249   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0 244 231 218 208 198 188 180 177 180 188 198 208 218 231 244   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0 242 229 216 203 190 180 170 164 170 180 190 203 216 229 242   0   0   0   0   0   0   0
0   0   0   0   0   0   0 255 242 229 216 203 190 177 164 151 164 177 190 203 216 229 242 255   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 164 170 180 190 203 216 229 242   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 177 180 188 198 208 218 231 244   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 190 190 198 206 216 226 236 249   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 203 203 208 216 224 234 244   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 216 216 218 226 234 242 252   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 229 229 231 236 244 252   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 242 242 244 249   0   0   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0 255   0   0   0   0   0   0   0   0   0   0   0   0   0   0
0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
esempio-dati
#endif
