#include <stdio.h>
#include <stdlib.h>

// matrice quadrata
typedef struct {
	int nrig;
	unsigned int **mat;
} matrice, *pmatrice;

#define TEST(X)	do { if((X)==NULL) { fprintf(stderr, "TEST failed: function %s, line %d\n", __func__, __LINE__); exit(1); } } while(0)

void mat_alloca(pmatrice m)
{
	TEST(m->mat = malloc(m->nrig * sizeof(*m->mat)));
	TEST(m->mat[0] = malloc(m->nrig * m->nrig * sizeof(**m->mat)));
	int i;
	for (i = 1; i < m->nrig; i++) {
    	m->mat[i] = m->mat[i-1]+m->nrig;
	}
}

void mat_stampa(pmatrice mat, FILE* f)
{
	int i, j;
	fprintf(f, "%d\n", mat->nrig);
	for(i=0; i<mat->nrig; i++) {
		for(j=0; j<mat->nrig; j++) {
			fprintf(f, "%2d ", mat->mat[i][j]);
		}
		fprintf(f, "\n");
	}
	fprintf(f, "\n");
}

void mat_leggi(pmatrice mat, FILE* f)
{
	int i, j;
	fscanf(f, "%d", &mat->nrig);
	mat_alloca(mat);
	for(i=0; i<mat->nrig; i++) {
		for(j=0; j<=i; j++) {
			fscanf(f, "%ud ", &mat->mat[i][j]);
		}
	}
	for(i=0; i<mat->nrig; i++) {
		for(j=i+1; j<mat->nrig; j++) {
			mat->mat[i][j] = mat->mat[j][i];
		}
	}
}

void mat_leggi_da_file(pmatrice mat, char *fn)
{
	FILE *f = fopen(fn, "r");
	if(!f) {
		perror(fn);
		exit(1);
	}
	mat_leggi(mat, f);
}

void mat_stampa_su_file(pmatrice mat, char *fn)
{
	FILE *f = fopen(fn, "w");
	if(!f) {
		perror(fn);
		exit(1);
	}
	mat_stampa(mat, f);
}

int cmp(int *a1, int *a2, int l)
{
	for(;l>0; l--, a1++, a2++) {
		if(*a1>*a2) return -1;
		if(*a1<*a2) return 1;
	}
	return 0;
}

double* media(pmatrice mat)
{
	int i, j;
	double *medie = calloc(mat->nrig+1, sizeof(double));
	for(i=0; i<mat->nrig; i++) {
		for(j=0; j<mat->nrig; j++) {
			medie[i] += mat->mat[i][j];
		}
		medie[mat->nrig] += medie[i];
		medie[i] /= mat->nrig;
	}
	medie[mat->nrig] /= (mat->nrig*mat->nrig);
	return medie;
}

void stampa_medie(pmatrice mat)
{
	double *medie = media(mat);
	int i;
	for(i=0; i<mat->nrig; i++) {
		printf("Media riga %d: %f\n", i, medie[i]);
	}
	printf("Media complessiva: %f\n", medie[mat->nrig]);
}

/*
 * ho riscritto la matrice, i vecchi dati sono persi
 * attenzione rischio di modificare dati che mi possono ancora servire
 */
void mat_nuova(pmatrice mat)
{
	unsigned int **m = mat->mat;
	mat->nrig/=2;
	int i, j;
	for(i=0; i<mat->nrig; i++) {
		for(j=0; j<mat->nrig; j++) {
			m[i][j] = (m[i*2][j*2]+m[i*2][j*2+1]+m[i*2+1][j*2]+m[i*2+1][j*2+1])/4;
		}
	}
}

int main(int argc, char *argv[])
{
	matrice mat;
	if(argc>1) {
		mat_leggi_da_file(&mat, argv[1]);
	} else {
		mat_leggi(&mat, stdin);
	}
	mat_stampa(&mat, stderr);
	stampa_medie(&mat);
	mat_nuova(&mat);
	if(argc>2) {
		mat_stampa_su_file(&mat, argv[2]);
	} else {
		mat_stampa(&mat, stdout);
	}
	return 0;
}

#if 0
esempio-dati
8
51
77 36
46 17 51
89 55 20 82
80 57 56 28 31
41 15 72 18 69 25
58 81 43 60 22 96  9
97 04 87 36 51 98 76 73
esempio-dati
#endif
